@@ -6,16 +6,15 @@ strat4 <- strat(cloud, c1 = "tenkan", c2 = "kijun", dir = "short")
 
 test_that("strat ok", {
   expect_s3_class(strat, "ichimoku")
-  expect_true(dim(strat)[2L] == 19L)
+  expect_identical(dim(strat), c(281L, 19L))
   expect_identical(strat, strat2)
   expect_error(strat(sample_ohlc_data), regexp = "ichimoku object")
   expect_error(strat(cloud, "close", "tenkan", "kijun", "chikou", type = "a"), regexp = "type invalid")
 })
 
 test_that("stratcombine ok", {
-  comb <- stratcombine(strat, strat3)
-  expect_s3_class(comb, "ichimoku")
-  expect_true(dim(comb)[2L] == 19L)
+  expect_s3_class(comb <- stratcombine(strat, strat3), "ichimoku")
+  expect_identical(dim(comb), c(281L, 19L))
   expect_identical(stratcombine(strat, strat), strat)
   expect_error(stratcombine(sample_ohlc_data), regexp = "ichimoku object")
   expect_error(stratcombine(strat, strat(cloud[1:100,])), regexp = "same data")
@@ -27,7 +26,13 @@ test_that("hasStrat ok", {
   expect_false(hasStrat(cloud))
 })
 
-test_that("summary.strat ok", {
-  expect_true(inherits(summary(strat), "matrix"))
-  expect_false(inherits(summary(strat, strat = FALSE), "matrix"))
+test_that("str method for strat ok", {
+  expect_output(expect_null(expect_invisible(str(strat))), "w/ strat")
+})
+
+test_that("summary method for strat ok", {
+  expect_type(summary(strat), "list")
+  expect_output(expect_vector(summary(strat, strat = FALSE), ptype = "character()"), "with dimensions")
+  attr(strat4, "strat") <- "invalid"
+  expect_output(summary(strat4), "invalid strategy")
 })
